#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <iostream>
#include <TROOT.h>
#include <TApplication.h>
#include <TSystem.h>
#include "foton.hh"
#include "filterwiz/TLGFilterWizard.hh"
#include "goption.hh"


   // ROOT object
   TROOT root ("GUI", "Foton");

   using namespace std;
   using namespace filterwiz;


   const char* const help_text =
   "Usage: foton [-w] [-d display] [-p path] [filename]\n"
   "       -w            read-write mode (readonly default)\n"
   "       -p 'path'     path of online configuration files\n"
   "       -d 'display'  X windows display\n"
   "       -h            this help";


//////////////////////////////////////////////////////////////////////////
//                                                                      //
// main                                                                 //
//                                                                      //
// Main Program                                                         //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
extern "C"
   int main (int argc, char** argv)
   {
      gdsbase::option_string opt (argc, argv, "vwp:d:h");
      // help
      if (opt.help() || (opt.argnum() > 1)) {
         cout << help_text << endl;
         exit (0);
      }
      // silence output if not verbose
      if (!opt.opt ('v')) {
         int i = ::open ("/dev/null", 2);
         (void) dup2 (i, 1);
         (void) dup2 (i, 2);   
      }
      // filename
      string filename;
      string shortname;
      string filepath;
      if (opt.argnum() == 1) {
         filename = *opt.argbegin();
         if (filename.find ("/") != string::npos) {
            filepath = gSystem->DirName (filename.c_str());
         }
         shortname = gSystem->BaseName (filename.c_str());
      }
      // path
      string path;
      opt.getOpt ('p', path);
      if ((!filepath.empty() && (filepath[0] == '/')) || path.empty()) {
         path = filepath;
      }
      else if (!filepath.empty()) {
         path += "/" + filepath;
      }
      // display
      string display;
      int rargc = 1;
      char* rargv[10];
      rargv[0] = argv[0];
      if (opt.getOpt ('d', display)) {
         rargv[1] = new char [12];
         strcpy (rargv[1], "-display");
         rargv[2] = new char [display.size() + 2];
         strcpy (rargv[2], display.c_str());
         rargc = 3;
      }
      // read-write mode
      bool rw_mode = opt.opt ('w');
   
      // initializing root environment
      TApplication theApp ("Foton", &rargc, rargv);
   
      // create main window
      bool modal = false;
      std::string fdesign;
      TLGFilterWizard mainWindow (gClient->GetRoot(), modal, "Foton", fdesign,
                           path.empty() ? 0 : path.c_str(), 
                           shortname.empty() ? 0 : shortname.c_str());
      mainWindow.Setup();
      mainWindow.SetReadOnly (!rw_mode);
      // run the GUI
      theApp.Run();
   
      return 0;
   }

