#ifndef LMSG_BUFFER_HH
#define LMSG_BUFFER_HH
#include "lmsg/MsgHeader.hh"

namespace lmsg {
  class BufferPool;
  class MsgHeader;

  /**  Socket I/O buffer
   */
  class Buffer {
  public:
    ~Buffer(void);
    Buffer(size_type size, char* data=0);

    //---------------------------------- Accessors
    const MsgHeader& getHeader(void) const;
    const char* getBuffer(void) const;
    const char* getDataAddr(void) const;
    size_type   getLength(void) const;
    size_type   getSize(void) const;
    size_type   getDataSize(void) const;
    size_type   getDataLength(void) const;
    void Return(void);

    //---------------------------------- Mutators
    char*       getBuffer(void);
    char*       getDataAddr(void);
    MsgHeader&  getHeader(void);
    void        setLength(size_type length);
    void        setDataLength(size_type length);
    void        setPool(BufferPool& pool);
  private:
    size_type   mSize;
    size_type   mLength;
    BufferPool* mPool;
    char*       mData;
    bool        mLocalData;
  }; // Buffer class
} // namespace lmsg

inline const char* 
lmsg::Buffer::getBuffer(void) const {
    return mData;
}

inline char*
lmsg::Buffer::getBuffer(void) {
    return mData;
}

inline const char* 
lmsg::Buffer::getDataAddr(void) const {
    return mData + sizeof(MsgHeader);
}

inline char*
lmsg::Buffer::getDataAddr(void) {
    return mData + sizeof(MsgHeader);
}

inline lmsg::size_type
lmsg::Buffer::getDataLength(void) const {
    return mLength - sizeof(MsgHeader);
}

inline lmsg::size_type
lmsg::Buffer::getDataSize(void) const {
    return mSize - sizeof(MsgHeader);
}

inline const lmsg::MsgHeader& 
lmsg::Buffer::getHeader(void) const {
    return *(const MsgHeader*)mData;
}

inline lmsg::MsgHeader& 
lmsg::Buffer::getHeader(void) {
    return *(MsgHeader*)mData;
}

inline lmsg::size_type
lmsg::Buffer::getLength(void) const {
    return mLength;
}

inline lmsg::size_type
lmsg::Buffer::getSize(void) const {
    return mSize;
}

#endif // LMSG_BUFFER_HH
