#ifndef Event_HH
#define Event_HH

#include "Time.hh"
#include "Interval.hh"
#include "TSeries.hh"

// header for simple event class

class Event {
public:

/** Default constructor
  */  
  Event(void);

/** Constructor with initial event timeseries
  */
  Event(TSeries* pts, Time event_time, 
	Interval event_duration, double event_amplitude );

/** Constructor with initial event timeseries for both filtered
  * and raw data 
  */
  Event(TSeries* ptseries, TSeries* prawtseries, Time event_time,
	Interval event_duration, double event_amplitude);

/** Copy constructor
 */
  Event(const Event&);

/** Assignment operator
 */
  Event& operator=(const Event&);

/** Default destructor
  */
  ~Event(void);

/** set fields for event excluding raw data
  */
  void setup(TSeries* ptseries, Time event_time,
	     Interval event_duration, double event_amplitude);

/** set fields for event including raw data
  */
  void setup(TSeries* ptseries, TSeries* prawtseries, Time event_time,
	     Interval event_duration, double event_amplitude );

/** Get the amplitude of an event
  */
  double getAmplitude(void); 

/** Get the time of an event
  */
  Time getTime(void);

/** Get time at start of event viewer
  */
  Time getViewerStartTime(void);

/** Get time period for event viewer
  */
  Interval getViewerDuration(void);

/** Get duration of an event
  */
  Interval getDuration(void);

/** Get timeseries of an event display
  */
  TSeries getDisplay(void);

/** Get pointer to timeseries for an event
  */
  TSeries* getDisplayPointer(void);

/** Add a raw display to an existing event
  */
  void addRawDisplay(TSeries* pts);

/** Get raw timeseries pointer for event
  */
  TSeries* getRawDisplayPointer(void);

/** Get raw timeseries for event
  */
  TSeries getRawDisplay(void);

/** Find out if raw data for the event is available
  */
  bool rawDataAvailable(void);

/** Write a text summary of an event to an output stream
  */
  void textSummary(std::ostream& os);

/** Write a short summary of an event to an output stream
  */
  void shortTextSummary(std::ostream& os);

/** Operator defining 'greater than'
  */
  bool operator>(const Event& y) const;

/** Operator defining 'less than'
  */
  bool operator<(const Event& y) const;

private:
  bool _in_use;
  bool _raw_available;
  TSeries* _event_display;
  TSeries* _raw_event_display;
  Time _event_time;
  Interval _event_duration;
  double _event_amplitude;
};

#endif // define Event_HH




