#ifndef SATSCAN_HH
#define SATSCAN_HH

#include "AlarmClient.hh"
#include "DatEnv.hh"
#include "LscPdBackout.hh"
#include "DynConfig.hh"
#include "MonServer.hh"
#include "OperStateCondList.hh"
#include "Trend.hh"
#include "FixedLenTS.hh"
#include <vector>
#include <string>

class SatChan : public LscPdBackout {
public:
  SatChan(const char* ifo, const char* port, int pd=-1, int N=4);
  ~SatChan(void);
  int getRepCount(void) const;
  const char* getOscCond() const;
  FixedLenTS& refHistoryI(void);
  FixedLenTS& refHistoryQ(void);
  void setRepCount(int N);
  void setOscCond(const char* cond);
private:
  int mRepCount;
  std::string mOscCond;
  FixedLenTS mHistoryI;
  FixedLenTS mHistoryQ;
};

inline
SatChan::SatChan(const char* ifo, const char* port, int pd, int N)
  : LscPdBackout(ifo, port, pd), mRepCount(N)
{}

inline
SatChan::~SatChan(void) {}

inline int
SatChan::getRepCount(void) const {
    return mRepCount;
}

inline FixedLenTS&
SatChan::refHistoryI(void) {
    return mHistoryI;
}

inline FixedLenTS&
SatChan::refHistoryQ(void) {
    return mHistoryQ;
}

inline const char*
SatChan::getOscCond(void) const {
    return mOscCond.c_str();
}

inline void
SatChan::setRepCount(int N) {
    mRepCount = N;
}

inline void
SatChan::setOscCond(const char* cond) {
    mOscCond = cond;
}

//======================================  SatScan monitor class
class SatScan : public DatEnv, MonServer {
public:
  SatScan(int argc, const char* argv[]);
  ~SatScan(void);
  void ProcessData(void);
  void ReadConfig(void);
  void Attention(void);
private:
  std::string mConfFile;
  std::string mOscFile;
  DynConfig   mConfig;
  typedef std::vector<SatChan> PD_List;
  typedef PD_List::iterator pd_iter;
  PD_List     mList;
  Time        mLast;
  int         mCount;
  OperStateCondList mOsc;
  Trend mTrend;
  AlarmClient mAlarm;
  AlarmData   mAlDat;
};

#endif // SATSCAN_HH




